# Copyrights Kevin Trancho 2021
# Web application using Flask to handle the database

import sqlite3
from flask import Flask, render_template, request, url_for, flash, redirect
from werkzeug.exceptions import abort
from itertools import *

def get_database_connexion():
	connexion = sqlite3.connect('database.db')
	connexion.row_factory = sqlite3.Row
	return connexion

def getClassStudents(connexion, cid):
	students = dict()
	students['liste'] = list(map(dict, connexion.execute('SELECT * FROM users WHERE classeId = ? ORDER BY (nom)',
			(cid,))))
	students['listeLen'] = len(list(students['liste']))
	students['liste'] = list(map(dict, connexion.execute('SELECT '
	'u.nom AS nom, '
	'u.prenom AS prenom, '
	'u.id AS userId, '
	'(SELECT COUNT(*) AS activityNumber FROM activity WHERE u.id = activity.userId) AS activityNumber '
	'FROM users u '
	'WHERE u.classeId = ? '
	'GROUP BY (u.id) '
	'ORDER BY (u.nom)',
			(cid, ))))
	students['activities'] = list(map(dict, connexion.execute('SELECT '
	'activity.sportId AS sportId, '
	'sports.nom AS nom, '
	'COUNT(*) AS activityNumber '
	'FROM users, activity, sports '
	'WHERE users.id = activity.userId '
	'AND activity.sportId = sports.id '
	'AND classeId = ? '
	'GROUP BY (activity.sportId) '
	'ORDER BY (activityNumber) DESC',
			(cid,))))
	return students

def getSportStudents(connexion, sid):
	data = dict()
	data['activity'] = list(map(dict, connexion.execute('SELECT * FROM activity WHERE sportId = ?', (sid,)).fetchall()))
	data['activityLen'] = len(data['activity'])
	data['users'] = list(map(dict, connexion.execute('SELECT '
	'users.nom AS nom, '
	'users.prenom AS prenom, '
	'users.id AS userId, '
	'classe.id AS classeId, '
	'classe.nom AS classeNom,'
	'COUNT(*) AS activityNumber '
	'FROM activity, users, classe '
	'WHERE activity.userId = users.id '
	'AND users.classeId = classe.id '
	'AND activity.sportId = ? '
	'GROUP BY activity.userId '
	'ORDER BY nom', (sid,)).fetchall()))
	data['usersLen'] = len(data['users'])
	data['shortestTime'] = list(map(dict, connexion.execute('SELECT '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.id AS userId, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = ? '
	'ORDER BY activityDuree', (sid,)).fetchall()))
	data['shortestDistance'] = list(map(dict, connexion.execute('SELECT '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.id AS userId, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = ? '
	'ORDER BY activityDistance', (sid,)).fetchall()))
	data['slowest'] = list(map(dict, connexion.execute('SELECT '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.id AS userId, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = ? '
	'ORDER BY activitySpeed', (sid,)).fetchall()))
	return data

app = Flask(__name__)
app.config['SECRET_KEY'] = 'Python is really magic'
	
@app.route('/')
def index() :
	connexion = get_database_connexion()
	connexion.close()
	return render_template("index.html")

@app.route('/about')
def about() :
	return render_template("about.html")

@app.route('/classes')
def classes() :
	connexion = get_database_connexion()
	classes = list(map(dict, connexion.execute('SELECT * FROM classe ORDER BY niveau DESC').fetchall()))
	students = dict()
	for c in classes :
		students[c['id']] = getClassStudents(connexion, c['id'])
	connexion.close()
	return render_template("classes.html", classes=classes, eleves=students)

@app.route('/classe/<int:id>')
def classe(id) :
	connexion = get_database_connexion()
	classe = dict(connexion.execute('SELECT * FROM classe WHERE id = ? ORDER BY niveau DESC', (id,)).fetchone())
	students = getClassStudents(connexion, id)
	activity = connexion.execute('SELECT '
	'activity.id AS activityId, '
	'activity.created AS activityCreated, '
	'activity.content AS activityContent, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'activity.commentaire AS activityCommentaire, '
	'activity.userId AS userId, '
	'activity.sportId AS sportId, '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.id AS userId, '
	'users.classeId AS classeId, '
	'sports.nom AS sportNom, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users, sports '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = sports.id '
	'AND users.classeId = ? '
	'ORDER BY activityId DESC', (id,)).fetchall()
	students['activity'] = list(map(dict, activity))
	connexion.close()
	return render_template("classe.html", classe=classe, eleves=students)

@app.route('/classeCreate', methods=('GET', 'POST'))
def createClasse():
	if request.method == 'POST':
		nom = request.form['nom']
		niveau = request.form['niveau']
		
		if not nom :
			flash("La classe doit avoir un nom")
		elif not niveau :
			flash("La classe doit avoir un niveau")
		else:
			connexion = get_database_connexion()
			connexion.execute('INSERT INTO classe (nom, niveau) VALUES (?, ?)',
				(nom, niveau))
			connexion.commit()
			connexion.close()
			return redirect(url_for('classes'))
	return render_template('classeCreate.html')

@app.route('/classeEdit/<int:id>', methods=('GET', 'POST'))
def editClasse(id):
	
	connexion = get_database_connexion()
	classe = dict(connexion.execute('SELECT * FROM classe WHERE id = ?', (id,)).fetchone())
	connexion.close()
	
	if request.method == 'POST':
		nom = request.form['nom']
		niveau = request.form['niveau']
		
		if not nom :
			flash("La classe doit avoir un nom")
		elif not niveau :
			flash("La classe doit avoir un niveau")
		else:
			connexion = get_database_connexion()
			connexion.execute('UPDATE classe SET nom = ?, niveau = ? WHERE id = ?',
				(nom, niveau, id))
			connexion.commit()
			connexion.close()
			return redirect(url_for('classes'))
	return render_template('classeEdit.html', classe=classe)

@app.route('/classeDelete/<int:id>', methods=('POST',))
def classeDelete(id):
	connexion = get_database_connexion()
	classe = dict(connexion.execute('SELECT * FROM classe WHERE id = ?', (id,)).fetchone())
	connexion.execute('DELETE FROM activity WHERE EXISTS (SELECT * FROM users u WHERE u.classeId = ? AND activity.userId = u.id)', (id,))
	connexion.execute('DELETE FROM users WHERE users.classeId = ?', (id,))
	connexion.execute('DELETE FROM classe WHERE id = ?', (id,))
	connexion.commit()
	connexion.close()
	flash('La classe {}, ses élèves et ses activités ne sont plus enregistrés sur Jean Rose Sport.'.format(classe['nom']))
	return redirect(url_for('classes'))

@app.route('/sports')
def sports() :
	data = dict()
	connexion = get_database_connexion()
	sports = list(map(dict, connexion.execute('SELECT * FROM sports ORDER BY (nom)').fetchall()))
	for s in sports :
		data[s['id']] = getSportStudents(connexion, s['id'])
	connexion.close()
	return render_template("sports.html", sports=sports, data=data)
	
@app.route('/sport/<int:id>')
def sport(id) :
	connexion = get_database_connexion()
	sport = dict(connexion.execute('SELECT * FROM sports WHERE id = ?', (id,)).fetchone())
	data = getSportStudents(connexion, id)
	activity = connexion.execute('SELECT '
	'activity.id AS activityId, '
	'activity.created AS activityCreated, '
	'activity.content AS activityContent, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'activity.commentaire AS activityCommentaire, '
	'activity.userId AS userId, '
	'activity.sportId AS sportId, '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.classeId AS classeId, '
	'classe.nom AS classeNom, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users, classe '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = ? '
	'AND users.classeId = classe.id '
	'ORDER BY activityId DESC', (id,)).fetchall()
	data['activity'] = list(map(dict, activity))
	connexion.close()
	return render_template("sport.html", sport=sport, data=data)

@app.route('/sportCreate', methods=('GET', 'POST'))
def createSport():
	if request.method == 'POST':
		nom = request.form['nom']
		
		if not nom :
			flash("Le sport doit avoir un nom")
		else:
			connexion = get_database_connexion()
			connexion.execute('INSERT INTO sports (nom) VALUES (?)',
				(nom, ))
			connexion.commit()
			connexion.close()
			return redirect(url_for('sports'))
	return render_template('sportCreate.html')

@app.route('/sportEdit/<int:id>', methods=('GET', 'POST'))
def editSport(id):
	
	connexion = get_database_connexion()
	sport = dict(connexion.execute('SELECT * FROM sports WHERE id = ?', (id,)).fetchone())
	connexion.close()
	
	if request.method == 'POST':
		nom = request.form['nom']
		
		if not nom :
			flash("Le sport doit avoir un nom")
		else:
			connexion = get_database_connexion()
			connexion.execute('UPDATE sports SET nom = ? WHERE id = ?',
				(nom, id))
			connexion.commit()
			connexion.close()
			return redirect(url_for('sports'))
	return render_template('sportEdit.html', sport=sport)

@app.route('/sportDelete/<int:id>', methods=('POST',))
def sportDelete(id):
	connexion = get_database_connexion()
	sport = dict(connexion.execute('SELECT * FROM sports WHERE id = ?', (id,)).fetchone())
	connexion.execute('DELETE FROM activity WHERE sportId = ?', (id,))
	connexion.execute('DELETE FROM sports WHERE id = ?', (id,))
	connexion.commit()
	connexion.close()
	flash('Le sport {} et ses activités ne sont plus enregistrés sur Jean Rose Sport.'.format(sport['nom']))
	return redirect(url_for('sports'))

@app.route('/students')
def students() :
	data = dict()
	connexion = get_database_connexion()
	eleves = connexion.execute('SELECT * FROM users ORDER BY (nom)').fetchall()
	data['elevesLen'] = len(list(connexion.execute('SELECT * FROM users ORDER BY (nom)').fetchall()))
	connexion.close()
	return render_template("students.html", eleves=eleves, data=data)

@app.route('/student/<int:id>')
def student(id):
	data = dict()
	connexion = get_database_connexion()
	data["eleve"] = dict(connexion.execute('SELECT '
	'users.nom AS nom, '
	'users.prenom AS prenom, '
	'users.id AS id, '
	'classe.id AS classeId, '
	'classe.nom AS classeNom '
	'FROM users, classe '
	'WHERE users.id = ? '
	'AND classe.id = users.classeId', (id,)).fetchone())
	activity = connexion.execute('SELECT '
	'activity.id AS activityId, '
	'activity.created AS activityCreated, '
	'activity.content AS activityContent, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'activity.commentaire AS activityCommentaire, '
	'activity.userId AS userId, '
	'activity.sportId AS sportId, '
	'sports.nom AS sportNom, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, sports '
	'WHERE activity.userId = ? '
	'AND activity.sportId = sports.id '
	'ORDER BY activityId DESC', (id,)).fetchall()
	data['activity'] = list(map(dict, activity))
	data['sports'] = list(map(dict, connexion.execute('SELECT '
	'CAST(ROUND(AVG(activity.duree)) AS INTEGER) AS activityDuree, '
	'CAST(ROUND(AVG(activity.distance)) AS INTEGER) AS activityDistance, '
	'COUNT(*) AS activityCount, '
	'activity.sportId AS sportId, '
	'sports.nom AS sportNom, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, sports '
	'WHERE activity.userId = ? '
	'AND activity.sportId = sports.id '
	'GROUP BY sports.id '
	'ORDER BY activityCount DESC', (id,)).fetchall()))
	data['displaySports'] = dict()
	for s, i in zip(data['sports'], count()) :
		s['index'] = i
		data['displaySports'][s['sportNom']] = list(map(dict, connexion.execute('SELECT '
		'activity.id AS activityId, '
		'activity.created AS activityCreated, '
		'activity.content AS activityContent, '
		'activity.duree AS activityDuree, '
		'activity.distance AS activityDistance, '
		'activity.commentaire AS activityCommentaire, '
		'activity.userId AS userId, '
		'activity.sportId AS sportId, '
		'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
		'FROM activity '
		'WHERE activity.userId = ? '
		'AND activity.sportId = ? '
		'ORDER BY activityId', (id, s['sportId'],)).fetchall()))
		for d, j in zip(data['displaySports'][s['sportNom']], count()):
			d['index'] = j
	connexion.close()
	return render_template("student.html", data=data)

@app.route('/studentCreate', methods=('GET', 'POST'))
def createStudent():
	
	connexion = get_database_connexion()
	data = dict()
	data["classes"] = list(map(dict, connexion.execute('SELECT * FROM classe ORDER BY niveau DESC').fetchall()))
	connexion.close()
	
	if request.method == 'POST':
		nom = request.form['nom'].upper()
		prenom = request.form['prenom']
		classeId = request.form['classeId']
		
		if not nom :
			flash("L'élève doit avoir un nom")
		elif not prenom :
			flash("L'élève doit avoir un prénom")
		elif not classeId :
			flash("L'élève doit avoir une classe")
		else:
			connexion = get_database_connexion()
			connexion.execute('INSERT INTO users (nom, prenom, classeId) VALUES (?, ?, ?)',
				(nom, prenom, classeId))
			connexion.commit()
			connexion.close()
			return redirect(url_for('students'))
	return render_template('studentCreate.html', data=data)

@app.route('/studentEdit/<int:id>', methods=('GET', 'POST'))
def editStudent(id):
	
	connexion = get_database_connexion()
	data = dict()
	data["eleve"] = dict(connexion.execute('SELECT * FROM users WHERE id = ?', (id,)).fetchone())
	data["classes"] = list(map(dict, connexion.execute('SELECT * FROM classe ORDER BY niveau DESC').fetchall()))
	connexion.close()
	
	if request.method == 'POST':
		nom = request.form['nom']
		prenom = request.form['prenom']
		classeId = request.form['classeId']
		
		if not nom :
			flash("L'élève doit avoir un nom")
		elif not prenom :
			flash("L'élève doit avoir un prénom")
		elif not classeId :
			flash("L'élève doit avoir une classe")
		else:
			connexion = get_database_connexion()
			connexion.execute('UPDATE users SET nom = ?, prenom = ?, classeId = ? WHERE id = ?',
				(nom, prenom, classeId, id))
			connexion.commit()
			connexion.close()
			return redirect(url_for('student', id=id))
	return render_template('studentEdit.html', data=data)

@app.route('/studentDelete/<int:id>', methods=('POST',))
def studentDelete(id):
	connexion = get_database_connexion()
	student = connexion.execute('SELECT * FROM users WHERE id = ?', (id,)).fetchone()
	connexion.execute('DELETE FROM users WHERE id = ?', (id,))
	connexion.execute('DELETE FROM activity WHERE userId = ?', (id,))
	connexion.commit()
	connexion.close()
	flash('{} {} et ses activités ne sont plus enregistrés sur Jean Rose Sport.'.format(student['prenom'], student['nom']))
	return redirect(url_for('students'))

@app.route('/createActivity/<int:studentId>', methods=('GET', 'POST'))
def createActivity(studentId):
	connexion = get_database_connexion()
	data = dict()
	data["eleve"] = dict(connexion.execute('SELECT * FROM users WHERE id = ?', (studentId,)).fetchone())
	data["sports"] = list(map(dict, connexion.execute('SELECT '
	'(SELECT COUNT(*) AS activityNumber FROM activity a WHERE s.id = a.sportId AND a.userId = ?) AS activityCount, '
	's.id AS sportId, '
	's.nom AS sportNom '
	'FROM sports s '
	'ORDER BY activityCount DESC', (studentId,)).fetchall()))
	connexion.close()
	
	if request.method == 'POST':
		sportId = request.form['sportId']
		distance = request.form['distance']
		heures = request.form['heures']
		minutes = request.form['minutes']
		secondes = request.form['secondes']
		
		if not sportId :
			flash("Une activité doit être liée à un sport")
		elif not distance :
			flash("Une distance est attendue")
		elif not heures or not minutes or not secondes :
			flash("Un temps doit être donné")
		else :
			temps = int(heures) * 3600 + int(minutes) * 60 + int(secondes)
			content = "{sportId:%r,temps:%r,duree:%r}" % (sportId, temps, distance)
			if temps <= 0 or int(distance) <= 0 :
				flash("Entrez une distance et un temps valide (positif et non nul)")
			else:
				connexion = get_database_connexion()
				connexion.execute('INSERT INTO activity (content, duree, distance, userId, sportId) VALUES (?, ?, ?, ?, ?)',
					(content, temps, distance, data['eleve']['id'], sportId))
				connexion.commit()
				connexion.close()
				return redirect(url_for('student', id=data['eleve']['id']))
		
	
	return render_template('activityCreate.html', data=data)

@app.route('/editActivity/<int:id>', methods=('GET', 'POST'))
def editActivity(id):
	connexion = get_database_connexion()
	data = dict()
	data["activity"] = dict(connexion.execute('SELECT * FROM activity WHERE id = ?', (id,)).fetchone())
	data["eleve"] = dict(connexion.execute('SELECT * FROM users WHERE id = ?', (data["activity"]["userId"],)).fetchone())
	data["sports"] = list(map(dict, connexion.execute('SELECT '
	'(SELECT COUNT(*) AS activityNumber FROM activity a WHERE s.id = a.sportId AND a.userId = ?) AS activityCount, '
	's.id AS sportId, '
	's.nom AS sportNom '
	'FROM sports s '
	'ORDER BY activityCount DESC', (data["activity"]["userId"],)).fetchall()))
	connexion.close()
	
	if request.method == 'POST':
		sportId = request.form['sportId']
		distance = request.form['distance']
		heures = request.form['heures']
		minutes = request.form['minutes']
		secondes = request.form['secondes']
		
		if not sportId :
			flash("Une activité doit être liée à un sport")
		elif not distance :
			flash("Une distance est attendue")
		elif not heures or not minutes or not secondes :
			flash("Un temps doit être donné")
		else :
			temps = int(heures) * 3600 + int(minutes) * 60 + int(secondes)
			content = "{sportId:%r,temps:%r,duree:%r}" % (sportId, temps, distance)
			if temps <= 0 or int(distance) <= 0 :
				flash("Entrez une distance et un temps valide (positif et non nul)")
			else:
				connexion = get_database_connexion()
				connexion.execute('UPDATE activity SET content = ?, duree = ?, distance = ?, userId = ?, sportId = ? WHERE id = ?',
					(content, temps, distance, data['eleve']['id'], sportId, id))
				connexion.commit()
				connexion.close()
				return redirect(url_for('student', id=data['eleve']['id']))
		
	
	return render_template('activityEdit.html', data=data)

@app.route('/activityDelete/<int:id>', methods=('POST',))
def activityDelete(id):
	connexion = get_database_connexion()
	act = connexion.execute('SELECT * FROM activity WHERE id = ?', (id,)).fetchone()
	connexion.execute('DELETE FROM activity WHERE id = ?', (id,))
	connexion.commit()
	connexion.close()
	flash('l\'activité a été effacée avec succès.')
	return redirect(url_for('student', id=act['userId']))

@app.route('/activity')
def activity() :
	connexion = get_database_connexion()
	activity = connexion.execute('SELECT '
	'activity.id AS activityId, '
	'activity.created AS activityCreated, '
	'activity.content AS activityContent, '
	'activity.duree AS activityDuree, '
	'activity.distance AS activityDistance, '
	'activity.commentaire AS activityCommentaire, '
	'activity.userId AS userId, '
	'activity.sportId AS sportId, '
	'users.nom AS userNom, '
	'users.prenom AS userPrenom, '
	'users.classeId AS classeId, '
	'classe.nom AS classeNom, '
	'sports.nom AS sportNom, '
	'ROUND(((1. * activity.distance) / activity.duree) * 3.6, 2) AS activitySpeed '
	'FROM activity, users, sports, classe '
	'WHERE activity.userId = users.id '
	'AND activity.sportId = sports.id '
	'AND users.classeId = classe.id '
	'ORDER BY activityId DESC').fetchall()
	connexion.close()
	activity = list(map(dict, activity))
	data = dict()
	data['activityLen'] = len(activity)
	return render_template("activity.html", activity=activity, data=data)