# Copyrights Kevin Trancho 2021
# Script for human data generation
# Interest : generate data base of human names

from random import *
from math import *

import matplotlib.pyplot as plt

def listOfAllMen():
	return [e for e in open("names/man").read().split("\n") if len(e) > 0]
	
def listOfAllWomen():
	return [e for e in open("names/woman").read().split("\n") if len(e) > 0]
	
def listOfAllLastNames():
	return [e for e in open("names/last").read().split("\n") if len(e) > 0]
	
menList = listOfAllMen()
womenList = listOfAllWomen()
lastList = listOfAllLastNames()

menSet = set(menList)
womenSet = set(womenList)
lastSet = set(lastList)

# Exponential law generation to fit last names observations in real life
def lastRandom(diff = 5, randomGen = random) :
	y = randomGen()
	x = - diff ** -1 * log(exp(- diff) * (1 - y) + y)
	return x
	
# Trial of normal law generation, but this have discontinuity in the average value
def normalRandom(bounded = True, diff = 1, randomGen = random) :
	y = randomGen()
	x = 0
	if bounded :
		x = sqrt(- diff ** -1 * log(exp(- diff) * (1 - y) + y))
	else :
		x = sqrt(- diff ** -1 * log(y))
	return x
	
# Box-Muller method for normal law generation
def boxMullerRandom(randomGen = random) :
	a = randomGen()
	b = randomGen()
	x1 = sqrt(-2 * log(a)) * cos(2 * pi * b)
	x2 = sqrt(-2 * log(a)) * sin(2 * pi * b)
	return (x1, x2)

def normalLaw(average = 0, scale = 1, **keys):
	sign = True if random() < 0.5 else False
	v = normalRandom(bounded = False, **keys)
	if sign : v *= -1
	return v * scale + average
	
def boxMullerLaw(average = 0, scale = 1, **keys):
	a, b = boxMullerRandom(**keys)
	return a * scale + average
	
def makeRandint(gen = random) :
	return lambda start, end, **keys : round(gen(**keys) * (end - start)) + start

lastRandint = makeRandint(lastRandom)
normalRandint = makeRandint(normalRandom)
	
def getManFirstName():
	return menList[randint(0, len(menList) - 1)]

def getWomanFirstName():
	return womenList[randint(0, len(womenList) - 1)]

def getLastName(gen = lambda s, e : lastRandint(s, e)):
	return lastList[gen(0, len(lastList) - 1)]

def getManHeight():
	return round(boxMullerLaw(175, 6))

def getWomanHeight():
	return round(boxMullerLaw(163, 5.5))
	
class Person :
	def __init__(self, male = None, firstNames = None, lastNames = None):
		self.male = male
		self.female = not male
		self.firstNames = firstNames
		self.firstName = firstNames[0]
		self.lastNames = lastNames
		self.lastName = "-".join(lastNames)
	
	def __repr__(self):
		res = "(%s" % self.firstName
		if len(self.firstNames) > 1 : res += " (, %s)" % ", ".join(self.firstNames[1 : ])
		res += " %s" % self.lastName
		res += ", %s" % ("homme" if self.male else "femme")
		res += ")"
		return res

def generatePerson():
	male = True if random() < 0.5 else False
	numberFirstNames = 3 - lastRandint(0, 2, diff = 10)
	numberLastNames = 1 + lastRandint(0, 1, diff = 4)
	firstNames = [getManFirstName() if male else getWomanFirstName() for i in range(numberFirstNames)]
	lastNames = [getLastName() for i in range(numberLastNames)]
	return Person(male, firstNames, lastNames)

if __name__ == "__main__":
	print("%d | %d man names" % (len(menList), len(menSet)))
	print("%d | %d woman names" % (len(womenList), len(womenSet)))
	print("%d | %d last names" % (len(lastList), len(lastSet)))
	
	#gen = random
	#gen = lastRandom
	#gen = normalRandom
	#gen = lambda : normalRandom(False)
	#gen = lambda : boxMullerRandom()[0]
	gen = getManHeight
	size = 100000
	genType = "int"
	
	l = [gen() for e in range(size)]
	print(min(l), max(l))
	if genType == "float" :
		n = 1000
		s = [0] * n
		for v in l : 
			x = int(v * n)
			if x < 0 or x >= len(s) : continue
			s[x] += 1
		for i in range(len(s)):
			s[i] = (i / n, s[i])
		
	else :
		s = [(e, l.count(e)) for e in set(l)]
	
	plt.plot(*list(zip(*s)), "-")
	plt.show()